package com.dzeio.openhealth.ui.dialogs

import android.app.AlertDialog
import android.app.DatePickerDialog
import android.app.Dialog
import android.content.res.Resources
import android.os.Build
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.MenuItem
import android.view.ViewGroup
import android.view.Window
import androidx.annotation.RequiresApi
import androidx.core.content.ContextCompat
import androidx.lifecycle.lifecycleScope
import androidx.navigation.fragment.findNavController
import androidx.navigation.fragment.navArgs
import com.dzeio.openhealth.R
import com.dzeio.openhealth.core.BaseFullscreenDialog
import com.dzeio.openhealth.data.weight.Weight
import com.dzeio.openhealth.databinding.DialogAddWeightBinding
import com.dzeio.openhealth.databinding.DialogEditWeightBinding
import com.dzeio.openhealth.ui.main.home.HomeViewModel
import com.google.android.material.datepicker.MaterialDatePicker
import com.google.android.material.dialog.MaterialAlertDialogBuilder
import com.google.android.material.timepicker.MaterialTimePicker
import dagger.hilt.android.AndroidEntryPoint
import kotlinx.coroutines.flow.collect
import java.util.*

@AndroidEntryPoint
class EditWeightDialog : BaseFullscreenDialog<HomeViewModel, DialogEditWeightBinding>(HomeViewModel::class.java) {

    override val bindingInflater: (LayoutInflater) -> DialogEditWeightBinding = DialogEditWeightBinding::inflate

    override val isFullscreenLayout = true

    val args: EditWeightDialogArgs by navArgs()

    lateinit var weight: Weight

//    override fun onBuilderInit(builder: AlertDialog.Builder) {
//        super.onBuilderInit(builder)
//
//        builder.apply {
//            setTitle("Add your weight (kg)")
//            setIcon(activity?.let { ContextCompat.getDrawable(it, R.drawable.ic_outline_timeline_24) })
//            setPositiveButton("Validate") { dialog, _ ->
//                save()
//            }
//            setNegativeButton("Cancel") { dialog, _ ->
//                dialog.cancel()
//            }
//
//        }
//    }

    override fun onDialogInit(dialog: Dialog) {
        super.onDialogInit(dialog)

        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE)
    }

    override fun onCreated(savedInstanceState: Bundle?) {
        super.onCreated(savedInstanceState)

        lifecycleScope.launchWhenStarted {
            viewModel.fetchWeight(args.id).collect {
                weight = it!!
                binding.layoutDialogEditWeightKg.value = it.weight.toInt()
                binding.layoutDialogEditWeightGram.value = ((it.weight - it.weight.toInt()) * 10 ).toInt()

                binding.layoutDialogEditWeightTimestamp.setText(it.formatTimestamp())
            }
        }

        binding.layoutDialogEditWeightKg.maxValue = 636
        binding.layoutDialogEditWeightKg.minValue = 0

        binding.layoutDialogEditWeightGram.maxValue = 9
        binding.layoutDialogEditWeightGram.minValue = 0

        binding.layoutDialogEditWeightTimestamp.setOnClickListener {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
                val date = Date(weight.timestamp)
                val datePicker = MaterialDatePicker.Builder.datePicker()
                    .setTitleText("Select Date")
                    .setSelection(weight.timestamp)
                    .build()

                val fragManager = requireActivity().supportFragmentManager

                datePicker.addOnPositiveButtonClickListener { tsp ->
                    val timePicker = MaterialTimePicker.Builder()
                        .setHour(date.hours)
                        .setMinute(date.minutes)
                        .setTitleText("Pouet")
                        .build()

                    timePicker.addOnPositiveButtonClickListener {
                        Log.d("T", "${timePicker.hour} ${timePicker.minute}")
                        val newDate = Date(tsp)
                        newDate.hours = timePicker.hour
                        newDate.minutes = timePicker.minute
                        weight.timestamp = newDate.time
                        binding.layoutDialogEditWeightTimestamp.setText(weight.formatTimestamp())
                        Log.d("Res", newDate.time.toString())
                    }
                    timePicker.show(fragManager, "dialog")
                }
                datePicker.show(fragManager, "dialog")
                Log.d("Tag", "${date.year + 1900}, ${date.month}, ${date.day}")
//                val dg = DatePickerDialog(requireActivity())
//                dg.setOnDateSetListener { _, year, month, day ->
//
//                }
//                dg.updateDate(date.year + 1900, date.month, date.day)
//                dg.show()
            } else {
                TODO("VERSION.SDK_INT < N")
            }


        }

    }

    private fun save() {
        lifecycleScope.launchWhenCreated {
            weight.weight = binding.layoutDialogEditWeightKg.value + (binding.layoutDialogEditWeightGram.value.toFloat() / 10)
            viewModel.addWeight(weight)
            findNavController().popBackStack()
        }
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        return when (item.itemId) {
            R.id.menu_fullscreen_dialog_save -> {
                save()
                true
            }
            R.id.menu_fullscreen_dialog_delete -> {
                MaterialAlertDialogBuilder(requireContext())
                    .setTitle("Delete Weight?")
                    .setMessage("Are you sure you want to delete this weight?")
                    .setPositiveButton("Yes") {_, _ ->
                        lifecycleScope.launchWhenStarted {
                            viewModel.deleteWeight(weight)
                            findNavController().popBackStack()
                        }
                    }
                    .setIcon(ContextCompat.getDrawable(requireContext(), R.drawable.ic_outline_delete_24))
                    .setNegativeButton("No") { _, _ ->}
                    .show()
                true
            }
            else -> super.onOptionsItemSelected(item)
        }

    }
}